; $LastChangedDate: 2008-05-21 21:12:12 -0500 (Wed, 21 May 2008) $
; $LastChangedRevision: 1902 $
; $HeadURL: svn://osiris/scheme-bos-clug/tags/1.07/bos-pres.scm $
(module bos-pres (lib "run.ss" "slideshow")
  
  (require (lib "code.ss" "slideshow"))
  
  (slide/title/center
   "A Basic Object System Using Macros"
   (page-para* "A Talk With Grant Rettke")
   (page-para* "")
   (page-para* "grettke@acm.org"))
  
  (slide/title
   "How To Roll A Basic Object System"
   (page-item "Features")
   'next      
   (page-subitem "Public Methods, Private Variables, No Inheritance (Simple huh?!)")
   'next
   (page-item "Goals")
   'next
   (page-subitem "Study Scheme, Macros, Language Constructs")
   'next
   (page-subitem "Chat About It With My Friends")
   'next
   (page-item "Approach")
   'next
   (page-subitem "Code First, Then Generation")   
   'next
   (page-subitem "Simplicity Trumps Efficiency")
   'next
   (page-subitem "No Mystery Code!"))
  
  (slide/title
   "Destination Code Features"
   'next
   (page-item "Primitive Object Implementation")
   'next
   (page-item "Methods and Variables")
   'next
   (page-item "Encapsulation")
   'next
   (page-item "Message Passing")
   'next
   (page-item "Duplicate variable/method name warning")
   'next
   (page-item "Built on \"Stock\" Language Features")
   (page-item "Reference: 5-prim-obj-stx-smpl.scm, 6-prim-obj-stx-smpl-tsts.scm"))
  
  (slide/title
   "Destination Code Sample"
   (code
    (define-object person
      (variables
       ([name #f]
        [age-years #f])
       (methods
        ([set-name (λ (arg) (set! name arg))]
         [get-name (λ () name)]
         [set-age-years (λ (arg) (set! age-years arg))]
         [get-age-years (λ () age-years)]      
         [age-in-days (λ () (* age-years 365))]
         [typed-name (λ () (cons (get-name) (class-name)))])))))
   (page-item "Brackets may be used anywhere parentheses are used")
   (page-subitem "Primarily to enhance readability")
   (page-item "(Getters and setters used to serve as the familiar; not Lisp style to do so)"))
  
  (slide/title
   "Destination Code Test Sample"
   (code
    (test-case
     "define-primitive-object"
     (let ([obj (person)])
       (check-not-false obj "creation")
       (obj 'set-age-years 27)
       (check-eq? 
        (obj 'get-age-years) 27 "get/set age-years")
       (obj 'set-name 'Joe)
       (check-eq? (obj 'get-name) 'Joe "get/set name")
       (check-eq? 
        (obj 'age-in-days) (* 27 365) "age-in-days")
       (check-eq? (obj 'class-name) 'person "class-name")
       (check-eq? 
        (length (obj 'method-names)) 10 "method-names")
       (check-eq? 
        'person (cdr (obj 'typed-name)) "typed-name")))))
  
  (slide/title
   "Step 1"
   (page-item "Exploring Primitive Language Features")
   'next
   (page-subitem "Object Creation")
   'next
   (page-subitem "Message Passing")
   'next
   (page-subitem "Lexical Scope")
   (page-item "Reference: 1-prm-feat.scm"))
  
  (slide/title
   "Object Creation"
   (code
    (define prim-obj-creation
      (λ () 
        (λ () 
          #t))))
   'next
   (page-item "Goal: An object is a thing that can be instantiated")
   'next
   (page-item "This code is a function that returns a 1st class function")
   'next
   (page-item "A 1st class function is a thing, virtually an \"object\"")
   'next
   (page-item "This is how objects will be instantiated in this system"))
  
  (slide/title
   "Message Passing"
   (code
    (define prim-obj-msg-passing
      (λ () 
        (λ (msg) 
          msg))))
   'next
   (page-item "Goal: An object is a thing that can receive a message")
   'next
   (page-item "The first class function above can receive messages")
   'next
   (page-item "This is how \"message-passing\" will occur in this object system"))
  
  (slide/title
   "Lexical Scope"
   (code
    (define prim-obj-lex-scope
      (λ () 
        (define x 11) (define y 12)
        (define frobnicate
          (λ (a b) 
            (+ a b)))
        (λ (msg) 
          (cons msg (frobnicate x y))))))
   'next 
   (page-item "Goal: Encapsulation, introduction of scope")
   'next 
   (page-item "Lambda introduces lexical scope for internal" (code define) "(aka" (code letrec) ") appearing immediately after it")
   'next 
   (page-item "The 1st class function returned by this function inherits the lexical scope in which it was created (" (code x) "," (code y) ", and" (code frobnicate) ")")
   'next
   (page-item "This is how objects with lexical scope and encapsulation are created"))
  
  (slide/title
   "prim-obj-lex-scope usage"
   (code
    (define obj (prim-obj-lex-scope))
    (obj 'my-message))
   (page-item "-> " (code (my-message . 23)))
   (page-item "This is how objects may be instantiated and sent messages"))
  
  (slide/title
   "The Non-Macro Primitive Object"
   'next
   (page-item "Combine those three primitive features to hand-code a primitive object")
   (page-item "Reference: 3-A-prim-obj.scm"))
  
  (slide/title
   "The Non-Macro Primitive Object Code Sample"
   (code
    (define person
      (λ () 
        (define age-years #f)
        (define set-age-years 
          (λ (arg) (set! age-years arg)))
        (define get-age-years (λ () age-years))
        (define age-in-days (λ () (* age-years 365)))
        (λ (msg . args) 
          (case msg
            [(set-age-years) (apply set-age-years args)]
            [(get-age-years) (get-age-years)]
            [(age-in-days) (age-in-days)]
            [else (error "message not understood" msg)])))))
   (page-item "The last λ expression is a closure which contains the lexical scope of the enclosing λ expression; this is how the \"object\" is created"))
  
  (slide/title
   "Next Step: On To Generation"
   'next
   (page-item "High Level Macro Review")
   'next
   (page-subitem "How much is worth discussing here?"))
  
  (slide/title
   "Macros 1 - Why"
   (bitmap "SE1.gif")
   'next
   (page-item "Modify input code to produce new output code")
   'next
   (page-item "Vastly superior to C style pre-processor macros")
   'next
   (page-item "Change shape, and even order of evaluation of, the code"))
  
  (slide/title
   "Macros 1 - Why - Example"
   'next
   (page-item "In a conditional expression, every clause may not be evaluated")
   'next
   (page-item "Consider a typical if-null-then check (illustrated by the macro" (code my-if) " moving forward)")
   'next
   (code
    (let ([fun null])
      (my-if (null? fun)
             (printf "Can't call x, it is null~n")
             (printf "x is ~a~n" (x)))))
   'next
   (page-item (code my-if) "could never be a function because it would evaluate its arguments, resulting in a null pointer")
   'next
   (page-item "Reference: macros.scm"))
  
  (slide/title
   "Macros 2 - What"
   (bitmap "SE2.gif")
   'next
   (page-item "The object sent to the macro is called a syntax object")
   'next
   (page-item "The macro itself is implemented by an object called a transformer"))
  
  (slide/title
   "Macros 2 - What - Example"
   'next
   (page-item "Syntax Object (everything enclosed by" (code my-if) ")")
   'next
   (code
    (my-if (null? fun)
           (printf "Can't call x, it is null~n")
           (printf "x is ~a~n" (x))))
   'next
   (page-item "Transformer (this is the" (code my-if) "macro)")
   'next
   (code
    [(_ clause true-body false-body)
     #'(let ([c clause])
         (if c 
             true-body
             false-body))])
   'next
   (page-item "#' is shorthand for surrounding the following shape inside a call to " (code syntax)))
  
  (slide/title
   "Macros 3 - How"
   'next
   (page-item "The macro" (code my-if) "takes an input form as its argument")
   'next
   (code
    (my-if (null? fun)
           (printf "Can't call x, it is null~n")
           (printf "x is ~a~n" (x))))
   'next
   (page-item "De-structures it using pattern matching into 3 different parts:" (code clause) ", " (code true-body) ", and" (code false-body))
   'next
   (code (my-if clause true-body false-body))
   'next
   (page-item "Defines a template for the new form (the resulting syntax object)")
   'next
   (code
    #'(let ([c clause]) (if c true-body false-body))))
  
  (slide/title
   "Macros 3 - How"
   'next
   (page-item "Expands the template by replacing the pattern variables with their actual values and environment, and returns the resulting syntax-object")
   'next
   (code
    (my-if (null? fun)
           (printf "Can't call x, it is null~n")
           (printf "x is ~a~n" (x))))
   'next
   (page-item "Expanding into")
   'next
   (code
    (let ((c (null? fun))) 
      (if c 
          (printf "Can't call x, it is null~n") 
          (printf "x is ~a~n" (x))))))
  
  (slide/title
   "Macros 3 - How - Visual"
   (bitmap "SE3.gif"))
  
  (slide/title
   "Two Kinds of Macros"
   'next
   (page-item "Hygienic")
   'next
   (page-subitem "Guarantee that expansion will not redefine existing name bindings")
   'next
   (page-item "Lexical Scope Twisting (Un-Hygienic)")
   'next
   (page-subitem "By design allows you to modify existing bindings")
   'next
   (page-subitem "Why? To introduce a" (code return) " statement, or see \"On Lisp\" Anaphoric Macros"))
  
  (slide/title
   "Hygienic Macro 'hm' Template Source"
   (code
    [(_ body)
     #'(begin
         (define food 'perch)
         (define utinsil 'fork)          
         (printf "~a, ~a~n" food utinsil)
         body)])
   (page-item (code _) "is the first argument of the pattern, and is always ignored." "Using" (code _) "is both loved and hated by Schemers")
   (page-item (code body) "matches the entire form appearing as the argument to" (code hm))
   (page-item "Everything following #' is the template"))
  
  (slide/title
   "Hygienic Macro 'hm' Template Body"
   (bitmap "SE4.gif")
   (code
    (let ([food 'salmon]
          [utinsil 'spoon])
      (hm 
       (printf 
        "~a, ~a~n" 
        food utinsil))))
   (page-item "Everything following" (code hm) ", along with its envrionment, is the argument for" (code hm)))
  
  (slide/title 
   "Hygienic Macro 'hm' Template Expansion" 
   (code
    (let ([food 'salmon]
          [utinsil 'spoon])
      (hm (printf "~a, ~a~n" food utinsil))))
   (page-item "Expands into")
   (code
    (let([food 'salmon] 
         [utinsil 'spoon])
      (begin 
        [define food 'perch] 
        [define utinsil 'fork] 
        (printf "~a, ~a~n" food utinsil) 
        (printf "~a, ~a~n" food utinsil))))
   (page-item "On the next page is the interesting part; the" (code printfs) "still use the correctly bound values"))
  
  (slide/title
   "Hygienic Macro 'hm' Template Expansion"
   (bitmap "HM.gif")
   (page-item "Prints \"perch, fork\", then \"salmon, spoon\""))
  
  (slide
   ; Commented out to conserve space
   ; "Syntactic Extension 5 - Hygienic Macro Template"
   (bitmap "SE5.gif"))
  
  (slide/title
   "Un-Hygienic Macro 'uhm' Template Source"
   (code
    [(_ body)
     (with-syntax 
         ([utinsil 
           (datum->syntax-object (syntax body) 'utinsil)])
       #'(begin
           (define food 'perch)
           (define utinsil 'fork)             
           (printf "~a, ~a~n" food utinsil)
           body))])
   'next
   (page-item (code with-syntax) "provides the functionality to twist the lexical scope within the macro")
   (page-item "In this macro," (code utinsil) "is inserted into the macro body's environment")
   (page-item "On the next page, you will see that the macro overrode the existing binding in the body"))
  
  (slide/title
   "Un-Hygienic Macro 'uhm' Template Expansion"
   (bitmap "UHM.gif")
   'next
   (page-item "Prints \"perch, fork\", then \"salmon, fork\"")
   'next   
   (page-item "The template inserts a new binding into the body for" (code utinsil) ", breaking hygiene"))
  
  (slide
   ; Commented out to conserve space   
   ; "Syntactic Extension 6 - Un-Hygienic Macro Template"
   (bitmap "SE6.gif"))
  
  (slide/title
   "Next Steps"
   'next
   (page-item "Implement a Primitive Object Syntax")
   'next
   (page-item "Implement Collision Detection")
   'next
   (page-item "Added Default Class Name & Methods Query"))
  
  (slide/title
   "Prim Obj Stx: Pattern"
   (code
    (syntax-case stx (variables methods)
      [(define-object name
         (variables ([v-name v-val] ...))
         (methods ([m-name m-body] ...)))]))
   (page-item "Reference: 4-prim-obj-stx.scm"))
  
  (slide/title
   "Prim Obj Stx: Template"
   (code
    #'(define name
        (λ () 
          (define class-name (λ () 'name))              
          (define method-names (λ () method-names-ls))
          (define method-names-ls 
            '(class-name method-names m-name ...))
          (define v-name v-val) ...
          (define m-name m-body) ...
          (λ (msg . args) 
            (case msg                  
              [(class-name) (class-name)]
              [(method-names) (method-names)]
              [(m-name) (apply m-name args)] ...
              [else 
               (raise 
                (string-append 
                 "message '" 
                 (symbol->string msg) 
                 "' not understood"))]))))))
  
  (slide/title
   "Prim Obj Stx: Support Code"
   'next
   (page-item "Macro Expander")
   'next
   (page-subitem "View the expanded code")
   'next
   (page-item "with-syntax")
   'next
   (page-subitem "Used to create unhygienic macros")
   'next
   (page-item "invalid/duplicate identifier detection")
   'next
   (page-subitem "Implementing using" (code identifier?) "and" (code bound-identifier=?)))  
  
  (slide/title
   "Thoughts"
   'next
   (page-item "Toys are for Learning")
   'next
   (page-item "The H-Word, and Other Hang-Ups")
   'next
   (page-item "Ideas Matter Most, Language Slavery, Innovation")
   'next
   (page-item "CoE: A Perfect \"First Time\"")
   'next
   (page-item "Thoughtful Teacher, Thoughtful Student")
   'next
   (page-item "As Difficult As [I] Make It")
   'next
   (page-item "The Midget vs. the Digits"))
  
  (slide/title
   "Resources"
   'next
   (page-item "The Scheme Programming Language, Third Edition. R. Kent Dybvig")
   'next
   (page-subitem "Inspiration for this task, the \"K&R\" book for Scheme")
   'next
   (page-item "PLT Scheme v372")
   'next
   (page-subitem "mzscheme, DrScheme, Documentation, Discussion List")
   'next
   (page-subitem "This presentation is written in Scheme, see bos-pres.scm and run.bat")
   'next
   (page-subitem "Hit F5 to evalute and work with any code in the REPL")
   'next
   (page-subitem "Use the module language. All code unit tested")
   'next
   (page-item "Web: Community-Scheme-Wiki, Schematics Scheme Cookbook")
   'next
   (page-item "[A history of] macro systems... http://lists.gnu.org /archive/html/chicken-users/2008-04/msg00013.html"))
  
  (slide/title
   "Version"
   (page-item "$LastChangedDate: 2008-05-21 21:12:12 -0500 (Wed, 21 May 2008) $")
   (page-item "$LastChangedRevision: 1902 $")
   (page-item "$HeadURL: svn://osiris/scheme-bos-clug/tags/1.07/bos-pres.scm $")))