; $LastChangedDate: 2008-05-21 20:24:12 -0500 (Wed, 21 May 2008) $
; $LastChangedRevision: 1896 $
; $HeadURL: svn://osiris/scheme-bos-clug/tags/1.07/macros.scm $
#|
"New syntactic extensions are defined by associating keywords with transformation 
procedures, or transformers. ... Transformers are created 
using syntax-rules or syntax-case, which allow transformations to be specified 
via pattern matching."

"Transformers destructure their input with syntax-case and rebuild their output 
with syntax."

"[Syntax case takes one argument]. The argument is a syntax object representing the
form to be processed. The return value is a syntax object representing the 
output form.'

"A syntax expression is like a quote expression except that the values of
pattern variables appearing within template are inserted into template, and
contextual information associated both with the input and with the template is
retained in the output to support lexical scoping."

    #' is the shorthand for syntax in PLT.

"(identifier? v) returns #t if v is a syntax object and (syntax-e stx) produces
 a symbol. (syntax-e stx) unwraps the immediate S-expression structure from a syntax
 object, leaving nested syntax structure (if any) in place. The result of 
 (syntax-e stx) is one of the following: a symbol, a syntax pair, the empty list,
 a vector containing syntax objects, some other kind of datum, usually a number, 
 boolean, or string"

    In other words (identifier? v) returns true if it was passed a symbol which is an 
    identifier.

"free-identifier=? is used to determine whether two identifiers would be equivalent if
they were to appear as free identifiers in the output of a transformer."

"bound-identifier=? is used to determine if two identifiers would be equivalent if they were to appear as bound identifiers in the output of a transformer."

"It is sometimes useful to construct a transformer's output in separate pieces, then 
put the pieces together. with-syntax facilitates this by allowing the creation of 
local pattern bindings."

(syntax-object->datum obj) 
"The procedure syntax-object->datum strips all syntactic information from a syntax object and returns the corresponding Scheme "datum."

(datum->syntax-object template-identifier obj) 
datum->syntax-object constructs a syntax object from obj that contains the same contextual information as template-identifier, with the effect that the syntax object behaves as if it were introduced into the code when template-identifier was introduced. The template identifier is often the keyword of an input form, extracted from the form, and the object is often a symbol naming an identifier to be constructed.
    
procedure: (generate-temporaries list)
returns: a list of distinct generated identifiers
"Transformers can introduce a fixed number of identifiers into their output by naming
each identifier. In some cases, however, the number of identifiers to be introduced
depends upon some characteristic of the input expression."

"If stx-expr expression does not produce a syntax object value, it is converted to one using datum->syntax-object with the lexical context of the expression."

|#

; identifier
(define-syntax test1
  (λ (stx) 
    (syntax-case stx ()
      [(_ val1)
       (printf "test1 Identifier? ~a~n" (identifier? #`val1))
       #'val1])))

(let ([x 10])
  (test1 'hi)
  (test1 x))

; free-identifier=?
(define-syntax test2
  (λ (stx) 
    (syntax-case stx ()
      [(_ val)
       (begin
         (printf "test2 identifier? ~a~n" (identifier? #'val))
         (printf "test2 free-identifier=? [vs. foo?] ~a~n" 
                 (and (identifier? #'val) (free-identifier=? #'val #'foo))))
       #''(done with test2)])))


(let ([baz 11])
  (test2 baz)
  (test2 foo))

; bound-identifier=?
(define-syntax test3
  (λ (stx) 
    (syntax-case stx ()
      [(_ val1 val2)
       (begin
         (printf "test3 identifier? val1 ~a~n" (identifier? #'val1))
         (printf "test3 identifier? val2 ~a~n" (identifier? #'val2))
         (printf "test3 bound-identifier=? ~a~n" 
                 (and (identifier? #'val1) 
                      (identifier? #'val2)
                      (bound-identifier=? #'val1 #'val2))))
       #''(done with test3)])))

(let ([baz 11])
  (test3 baz bar)
  (test3 foo foo))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(printf ";;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;~n")
(printf "This is the relevant bit for the presentation~n")
(printf ";;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;~n")

(define-syntax my-if
  (lambda (stx) 
    (syntax-case stx ()
      [(_ clause true-body false-body)
       #'(let ([c clause])
           (if c 
               true-body
               false-body))])))

(let((food 'salmon) 
     (utinsil 'spoon))
  (begin 
    (define food 'perch) 
    (define utinsil 'fork) 
    (printf "~a, ~a~n" food utinsil) 
    (printf "~a, ~a~n" food utinsil)))

(let ([fun null])
  (my-if (null? fun)
         (printf "Can't call x, it is null~n")
         (printf "x is ~a~n" (x))))

;hygienic scoping
(define-syntax hm
  (λ (stx) 
    (syntax-case stx ()
      [(_ body)
       #'(begin
           (define food 'perch)
           (define utinsil 'fork)          
           (printf "~a, ~a~n" food utinsil)
           body)])))

(let ([food 'salmon]
      [utinsil 'spoon])
  (hm (printf "~a, ~a~n" food utinsil)))

;unhygienic scoping
(define-syntax uhm
  (λ (stx) 
    (syntax-case stx ()
      [(_ body)
       (with-syntax ([utinsil (datum->syntax-object #'body 'utinsil)])
         #'(begin
             (define food 'perch)
             (define utinsil 'fork)             
             (printf "~a, ~a~n" food utinsil)
             body))])))

(let ([food 'salmon]
      [utinsil 'spoon])
  (uhm(printf "~a, ~a~n" food utinsil)))